@@ -83,7 +83,7 @@ default_styles=r"""
 import ROOT # allows access to ROOT colors (e.g. ROOT.kRed)
 
 
-## Place this file in your home directory as '~/.rootplot', and
+## Place this file in your home directory as '~/.rootplotrc', and
 ## it will be automatically loaded whenever you call rootplot.
 ## Otherwise, include it in the arguments to rootplot to use it.
 ## Comment-out/delete sections you don't want to deal with.
@@ -91,15 +91,15 @@ import ROOT # allows access to ROOT colors (e.g. ROOT.kRed)
 ##############################################################################
 ######## Advanced Operation ##################################################
 
-## You may specify a list of files along with names for the legend
-## When enabled, any filenames on the command line will be ignored.
-## filenames = [
-##     ('histTTbar.root', r'$\bar{t}t$'),
-##     ('histZmumu.root', r'$Z\rightarrow\mu\mu$'),
-##     ]
+## Any option available at the command-line can also be specified in the
+## configuration file.  Dashes in command names are translated to underscores
+## in the variable names.  Any commands that take a comma-separated list can
+## be specified here as python lists instead.
 
-## All scale factor are 1 by default; specify different scale factors here
-file_scale_factors = []  # must be as long as file list
+## You can specify the files to run on through the 'filenames' variable rather
+## than entering them at the command-line, for example:
+## filenames = ['histTTbar.root', 'histZmumu.root']
+## legend_entries = [r'$\bar{t}t$', r'$Z\rightarrow\mu\mu$']
 
 ##############################################################################
 ######## Global Style Options ################################################
@@ -168,7 +168,6 @@ ratio_min = None
 #### Titles produced by --area-normalize and --normalize
 area_normalized_title = 'Fraction of Events in Bin'
 file_normalized_title = 'Events Normalized to %s'
-scale_normalized_title = 'Events Scaled by %d'
 
 #### Overflow and underflow text labels
 overflow_text = ' Overflow'
@@ -307,22 +306,27 @@ next_counter = counter_generator().next
 
 ######## These functions are the meat of this program #########################
 
-#### A recursive function to drill down through directories
-def process_directory(path, files, options):
-    """Loop through all histograms in the directory and plot them."""
+def get_folders_and_objects(path, files, options):
+    """Generate lists of subdirectories and of full paths to histograms."""
+    ## Delete existing output directory if appropriate
     if path == '' and os.path.exists(options.output):
         shutil.rmtree(options.output)
         os.makedirs(options.output)
     dir_to_make = '%s/%s' % (options.plot_dir, path)
+    ## Loop over TKeys in this folder
     keys = files[0].file.GetDirectory(path).GetListOfKeys()
-    plots_to_make = []
+    folders, objects = [], []
     for key in keys:
         name = key.GetName()
         classname = key.GetClassName()
-        new_path = '%s/%s' % (path, name)
+        new_path = os.path.join(path, name)
+        ## Descend into subdirectory
         if 'TDirectory' in classname:
-            plots_to_make += process_directory(new_path, files, options)
-        #### If key is a 1D histogram, add it to the list
+            folders += [new_path]
+            results = get_folders_and_objects(new_path, files, options)
+            folders += results[0]
+            objects += results[1]
+        ## Add this object to the list if it matches our specifications
         matches_path = options.regex.search(new_path)
         dimension = 0
         if ('TH1' in classname or
@@ -334,13 +338,40 @@ def process_directory(path, files, options):
                               (dimension == 2 and options.draw2D))):
             if not os.path.exists(dir_to_make):
                 os.makedirs(dir_to_make)
-            counter = next_counter() # used for page numbers
-            plots_to_make += [(path, name, counter)]
-    return plots_to_make
+            objects += [os.path.join(path, name)]
+    return folders, objects
+
+def get_plot_recipes(files, options):
+    """
+    Return a recipe list of plots to overlay.
+
+    The returned list items are of the form: ([(filenum, path), ...], counter).
+    """
+    folders, objects = get_folders_and_objects('', files, options)
+    recipes = []
+    if options.compare:
+        paths = options.compare
+        if paths[0] in folders:
+            reg = re.compile(r'%s/.*' % paths[0])
+            plot_names = [os.path.basename(x) for x in objects if reg.match(x)]
+            for name in plot_names:
+                recipes.append([(0, os.path.join(path, x)) for path in paths])
+            if not options.legend_entries:
+                options.legend_entries = paths
+        else:
+            recipes.append(zip([0] * len(paths), paths))
+            if not options.legend_entries:
+                options.legend_entries = paths
+    else:
+        nfiles = options.nfiles
+        for obj in objects:
+            recipes.append(zip(range(nfiles), [obj] * nfiles))
+        if not options.legend_entries:
+            options.legend_entries = [name[:-5] for name in options.filenames]
+    return [(file_path_pair, next_counter()) for file_path_pair in recipes]
 
-#### Handle multiprocessing
 def plotting_worker(queue, queue_globals, options):
-    """Handle a single plot, communicating with the queue."""
+    """Create a single plot, communicating with the queue."""
     while True:
         try:
             args = queue.get(timeout=5)
@@ -353,15 +384,21 @@ def plotting_worker(queue, queue_globals, options):
         queue.task_done()
 
 #### This is where all the plotting actually happens
-def process_hist(path, name, counter, options):
-    """Overlay all the instances of this plot."""
-    files = open_rootfiles(options.filenames)
-    options = process_local_options(options, name)
+def process_hist(filenum_path_pairs, counter, options):
+    """Create a plot from the hists indicated by filenum_path_pairs."""
+    files = open_rootfiles(options)
+    paths = [path for i, path in filenum_path_pairs]
+    for savepath in paths:
+        if savepath != paths[0]:
+            savepath = "%.3i" % counter
+            break
+    options = process_local_options(options, path)
     hists = []
     #### Apply options to hist from each file
-    for i, file in enumerate(files):
+    for i, (filenum, path) in enumerate(filenum_path_pairs):
+        thisfile = files[filenum]
         try:
-            roothist = file.file.GetDirectory(path).Get(name)
+            roothist = thisfile.file.Get(path)
         except ReferenceError:
             roothist = None
         if not roothist:
@@ -371,25 +408,23 @@ def process_hist(path, name, counter, options):
         dimension = 1
         if not isTGraph:
             dimension = roothist.GetDimension()
-            roothist.Scale(options.file_scale_factors[i])
+            roothist.Scale(options.scale[i])
         title, xlabel, ylabel = get_labels(roothist, options)
         if options.normalize:
-            if type(options.normalize) is int:
-                file_name = files[options.normalize - 1].name
-                ylabel = options.file_normalized_title % file_name
-            else:
-                ylabel = options.scale_normalized_title % options.normalize
+            file_name = files[options.normalize - 1].name
+            ylabel = options.file_normalized_title % file_name
         if options.area_normalize:
             ylabel = options.area_normalized_title
         if dimension == 1:
-            hist = r2m.Hist(roothist, replace=options.replace, label=file.name,
-                            title=title, xlabel=xlabel, ylabel=ylabel)
+            hist = r2m.Hist(roothist, replace=options.replace,
+                            label=thisfile.name, title=title,
+                            xlabel=xlabel, ylabel=ylabel)
         else:
             hist = r2m.Hist2D(roothist, replace=options.replace,
-                              label=file.name, title=title,
+                              label=thisfile.name, title=title,
                               xlabel=xlabel, ylabel=ylabel)
         if options.efficiency_from:
-            denom = r2m.Hist(file.file.Get(options.efficiency_from))
+            denom = r2m.Hist(thisfile.file.Get(options.efficiency_from))
             hist.y, up, down = wilson(hist.y, denom.y)
             hist.yerr = [down, up]
         hists.append(hist)
@@ -404,31 +439,27 @@ def process_hist(path, name, counter, options):
                     hist.scale(1./sum(hist.y))
             if options.normalize:
                 numerhist = hists[options.normalize - 1]
-                if type(options.normalize) is int:
-                    hists[options.normalize - 1]
-                    if options.range:
-                        lowbin, highbin = parse_range(hist.xedges,
-                                                      options.range)
-                        numer = numerhist.TH1F().Integral(lowbin, highbin)
-                        denom = hist.TH1F().Integral(lowbin, highbin)
-                    else:
-                        numer = sum(numerhist.y)
-                        denom = sum(hist.y)
-                    if denom:
-                        hist.scale(numer / denom)
+                hists[options.normalize - 1]
+                if options.range:
+                    lowbin, highbin = parse_range(hist.xedges,
+                                                  options.range)
+                    numer = numerhist.TH1F().Integral(lowbin, highbin)
+                    denom = hist.TH1F().Integral(lowbin, highbin)
                 else:
-                    hist.scale(options.normalize)
+                    numer = sum(numerhist.y)
+                    denom = sum(hist.y)
+                if denom:
+                    hist.scale(numer / denom)
     if options.mpl:
-        process_hist_mpl(path, name, counter, options, hists)
+        process_hist_mpl(savepath, counter, options, hists)
     else:
-        process_hist_root(path, name, counter, options, hists,
+        process_hist_root(savepath, counter, options, hists,
                           title, xlabel, ylabel, isTGraph)
 
 
-def process_hist_root(path, name, counter, options, hists,
+def process_hist_root(savepath, counter, options, hists,
                       title, xlabel, ylabel, isTGraph):
     """Overlay all the instances of this plot in ROOT and apply the options."""
-    global canvas
     canvas = ROOT.TCanvas()
     if options.ratio and not isTGraph:
         pads = divide_canvas(canvas, options.ratio_fraction)
@@ -493,15 +524,12 @@ def process_hist_root(path, name, counter, options, hists,
                     roothist.Draw("same " + options.drawcommands_root[i])
         else:
             roothist.Draw(options.draw2D)
-        try:
-            legend.AddEntry(roothist, options.filenames[i][:-5])
-        except TypeError:
-            legend.AddEntry(roothist, options.filenames[i][1])
+        legend.AddEntry(roothist, options.legend_entries[i])
     if isTGraph:
         multigraph.Draw(options.draw_graph)
     if options.ratio and dimension == 1 and not isTGraph:
         pads[1].cd()
-        stack_ratio = plot_ratio_root(hists, xlabel, options)
+        ratio_multigraph = plot_ratio_root(hists, xlabel, options)
         pads[0].cd()
     if options.logx:
         for pad in pads:
@@ -528,11 +556,11 @@ def process_hist_root(path, name, counter, options, hists,
         a_label.SetBorderSize(0)
         a_label.SetFillStyle(0)
         a_label.Draw()
-    save_plot(options.plot_dir, path, name, options.ext, counter)
+    save_plot(canvas, options.plot_dir, savepath, options.ext, counter)
     del canvas
     
 
-def process_hist_mpl(path, name, counter, options, hists):
+def process_hist_mpl(path, counter, options, hists):
     """Overlay all the instances of this plot and output using matplotlib."""
     fig = plt.figure(1, figsize=parse_size(options.size))
     plt.clf() # clear figure
@@ -651,7 +679,7 @@ def process_hist_mpl(path, name, counter, options, hists):
         plt.figtext(options.numbering_x_mpl, options.numbering_y_mpl,
                     counter, size=options.numbering_size_mpl,
                     ha=options.numbering_ha_mpl, va=options.numbering_va_mpl)
-    output_file_name = '%s/%s/%s' % (options.plot_dir, path, name)
+    output_file_name = '%s/%s' % (options.plot_dir, path)
     try:
         plt.savefig(output_file_name, transparent=options.transparent,
                     dpi=options.dpi)
@@ -663,13 +691,9 @@ def process_hist_mpl(path, name, counter, options, hists):
 
 ######## Define some supporting functions #####################################
 
-def open_rootfiles(filenames):
+def open_rootfiles(options):
     """Return a list of RootFile objects from a list of file names."""
-    try:
-        return [r2m.RootFile(name) for name in filenames]
-    except TypeError:
-        return [r2m.RootFile(name, displayname)
-                for name, displayname in filenames]
+    return [r2m.RootFile(options.filenames[i]) for i in range(options.nfiles)]
 
 def parse_size(size_option):
     """Return a width and height parsed from size_option."""
@@ -727,9 +751,9 @@ def get_labels(hist, options):
             ylabel = options.ylabel
     return title, xlabel, ylabel
 
-def save_plot(plot_dir, path, name, ext, counter):
+def save_plot(canvas, plot_dir, path, ext, counter):
     """Save the canvas to the output format defined by --ext."""
-    output_file_name = '%s/%s/%s.%s' % (plot_dir, path, name, ext)
+    output_file_name = '%s/%s.%s' % (plot_dir, path, ext)
     canvas.SaveAs(output_file_name)
 
 def report_progress(counter, nplots, output, ext, divisor=1):
@@ -739,19 +763,25 @@ def report_progress(counter, nplots, output, ext, divisor=1):
               (counter, nplots, output, ext)),
         sys.stdout.flush()
 
-def merge_pdf(output, plots_to_make):
+def merge_pdf(output, recipes):
     """Merge together all the produced plots into one pdf file."""
-    if not plots_to_make:
+    if not recipes:
         print "No output files, so no merged pdf was made"
         return
     print "Writing %s.pdf..." % output
-    paths = ["%s%s/%s" % (output, path, name)
-             for (path, name, counter) in plots_to_make]
-    paths = [x.replace(' ', r'\ ') for x in paths]
+    pdfpaths = []
+    for (filenum_path_pairs, counter) in recipes:
+        paths = [path for i, path in filenum_path_pairs]
+        for savepath in paths:
+            if savepath != paths[0]:
+                savepath = "%.3i" % counter
+                break
+        savepath = os.path.join(output, savepath).replace(' ', r'\ ')
+        pdfpaths.append(savepath)
     os.system('gs -q -dBATCH -dNOPAUSE -sDEVICE=pdfwrite '
               '-dAutoRotatePages=/All '
               '-sOutputFile=%s.pdf ' % output +
-              '.pdf '.join(paths) + '.pdf')
+              '.pdf '.join(pdfpaths) + '.pdf')
 
 def display_page_number(page_number, options):
     """Add a page number to the top corner of the canvas."""
@@ -843,11 +873,6 @@ def plot_ratio_root(hists, xlabel, options):
             ratio_hist = hist.divide_wilson(denom)
         else:
             ratio_hist = hist.divide(denom)
-        ## average = ratio_hist.Integral() / ratio_hist.GetNbinsX()
-        ## for j in range(ratio_hist.GetNbinsX()):
-        ##     if ratio_hist.GetBinContent(j) > options.ratio_cutoff * average:
-        ##         ratio_hist.SetBinContent(j, 0)
-        ##         ratio_hist.SetBinError(j, 0)
         graph = ratio_hist.TGraph()
         graph.SetLineColor(options.linecolors[i])
         graph.SetMarkerColor(options.markercolors[i])
@@ -920,25 +945,23 @@ def wilson(numerator, denominator):
         down.append(eff[i] - (1/(1 + 1/n) * (eff[i] + 1/(2*n) - sqrt)))
     return eff, up, down
 
-def make_html_index(base_dir, directory, filetype, stylesheet, ncolumns=2):
-    fullpath = '%s%s' % (base_dir, directory)
-    if not os.path.exists(fullpath):
-        os.makedirs(fullpath)
-    output = open('%s/index.html' % fullpath, 'w')
-    contents = sorted(os.listdir(fullpath))
-    subdirs = fullpath.split('/')
-    pathnav = '/'.join(['<a href="%s">%s</a>' %
-                        ('../' * (len(subdirs) - i - 1), subdirs[i])
-                        for i in range(len(subdirs))])
-    nav_items = ['<li><a href="%s">%s</a></li>' % (x, x) for x in contents
-                 if os.path.isdir("%s/%s/%s" % (base_dir, directory, x))]
-    navigation = '\n    '.join(nav_items)
-    img_items = ['<a name="%s"><a href="index.html#%s"><img src="%s"></a></a>'
-                 % (x,x,x) for x in contents if x.endswith(filetype)]
-    image_markup = '\n' + '\n'.join(img_items)
-    image_markup = re.sub('((\\n<img.*){%i})' % ncolumns,
-                          r'\1<br>', image_markup)
-    output.write(
+def make_html_index(directory, dirs, files, filetype, stylesheet, ncolumns=2):
+    output_file = open(os.path.join(directory, 'index.html'), 'w')
+    previous_dirs = directory.split('/')
+    ndirs = len(previous_dirs)
+    back_nav = ['<a href="%s">%s</a>' %
+                ('../' * (ndirs - i - 1) + 'index.html', previous_dirs[i])
+                for i in range(ndirs)]
+    back_nav = '/'.join(back_nav) + '/'
+    forward_nav = ['<li><a href="%s/index.html">%s/</a></li>' % (x, x)
+                   for x in dirs]
+    forward_nav = '\n    '.join(forward_nav)
+    plots = ['<a name="%s"><a href="index.html#%s"><img src="%s"></a></a>'
+             % (x,x,x) for x in files if x.endswith(filetype)]
+    plots = '\n' + '\n'.join(plots)
+    plots = re.sub('((\\n<img.*){%i})' % ncolumns,
+                   r'\1<br>', plots)
+    output_file.write(
         '''
         <html>
         <head>
@@ -949,14 +972,13 @@ def make_html_index(base_dir, directory, filetype, stylesheet, ncolumns=2):
         <h1>Navigation</h1>
           %s
           <ul>
-            <li><a href="..">..</a></li>
               %s
           </ul>
         <h1>Images</h1>
         %s
         </body>
-        </html>''' % (stylesheet, directory, pathnav, navigation, image_markup))
-    output.close()
+        </html>''' % (stylesheet, directory, back_nav, forward_nav, plots))
+    output_file.close()
 
 def parse_range(xedges, expression):
     """Returns the indices of the low and high bins indicated in expression."""
@@ -969,7 +991,7 @@ def parse_range(xedges, expression):
 
 def parse_legend_root(options):
     """Return the corners to use for the legend based on options."""
-    legend_height = min(options.legend_entry_height * options.nfiles + 0.02,
+    legend_height = min(options.legend_entry_height * options.nhists + 0.02,
                         options.max_legend_height)
     if type(options.legend) is int:
         options.legend = options.legend_codes[options.legend]
@@ -1020,9 +1042,9 @@ def add_options_from_configuration(options):
         sys.exit(0)
     import default_rootplot_config
     options = append_to_options(default_rootplot_config, options)
-    rc_path = os.path.expanduser('~/.rootplot')
+    rc_path = os.path.expanduser('~/.rootplotrc')
     if os.path.exists(rc_path):
-        print "Using styles and options from ~/.rootplot"
+        print "Using styles and options from ~/.rootplotrc"
         shutil.copy(rc_path, configdir + '/rootplotrc.py')
         options.config_files.insert(0, 'rootplotrc.py')
     for f in options.config_files:
@@ -1038,7 +1060,17 @@ def add_options_from_configuration(options):
 def process_options(options):
     """Select plot-level options based on the user-level input."""
     options.nfiles = nfiles = len(options.filenames)
-    if nfiles > 1: options.draw2D = None
+    options.nhists = nhists = len(options.filenames)
+    if options.compare:
+        options.compare = options.compare.split(',')
+        options.nhists = nhists = len(options.compare)
+        if nfiles != 1:
+            print("Cannot take more than one input file with the --compare "
+                  "option")
+            sys.exit(1)
+    if type(options.legend_entries) is str:
+        options.legend_entries = options.legend_entries.split(',')
+    if nhists > 1: options.draw2D = None
     if options.ratio_eff: options.ratio = options.ratio_eff
     if options.ratio_eff: options.ratio_max = 1.
     if options.merge: options.ext = 'pdf'
@@ -1068,13 +1100,13 @@ def process_options(options):
             from matplotlib import pyplot as plt
         if options.colormap:
             cmap = plt.get_cmap(options.colormap)
-            if options.ncolors < nfiles:
-                options.ncolors = nfiles
+            if options.ncolors < nhists:
+                options.ncolors = nhists
             options.colors = [cmap(float(i)/options.ncolors)
-                              for i in range(nfiles)]
+                              for i in range(nhists)]
         else:
             options.ncolors = len(options.colors)
-            if nfiles > options.ncolors:
+            if nhists > options.ncolors:
                 print "Not enough colors defined!"
                 sys.exit(1)
         if options.mpl:
@@ -1084,50 +1116,45 @@ def process_options(options):
                 print("Unable to load numpy, "
                       "which is required for operation with matplotlib")
                 sys.exit(1)
-    if options.normalize:
-        if '.' in options.normalize:
-            options.normalize = float(options.normalize)
-        else:
-            options.normalize = int(options.normalize)
     if options.efficiency_from:
         options.efficiency = True
-    if not options.file_scale_factors:
-        options.file_scale_factors = [1.0 for i in xrange(nfiles)]
+    if type(options.scale) is not list:
+        options.scale = [options.scale for i in xrange(nhists)]
     if not options.markers:
-        options.markers_mpl = ['o' for i in xrange(nfiles)]
+        options.markers_mpl = ['o' for i in xrange(nhists)]
     if not options.linecolors:
         options.linecolors = options.colors
     if not options.fillcolors:
         options.fillcolors = options.colors
     if not options.linestyles_mpl:
-        options.linestyles_mpl = ['solid' for i in xrange(nfiles)]
+        options.linestyles_mpl = ['solid' for i in xrange(nhists)]
     if not options.linestyles_root:
-        options.linestyles_root = [1 for i in xrange(nfiles)]
+        options.linestyles_root = [1 for i in xrange(nhists)]
     if not options.fillstyles_root:
         if options.fill or options.data:
-            options.fillstyles_root = [1001 for i in xrange(nfiles)]
+            options.fillstyles_root = [1001 for i in xrange(nhists)]
         else:
-            options.fillstyles_root = [0 for i in xrange(nfiles)]
+            options.fillstyles_root = [0 for i in xrange(nhists)]
     if not options.markercolors:
         options.markercolors = options.colors
     if not options.plotstyles_mpl:
-        options.plotstyles_mpl = [options.mpl for i in xrange(nfiles)]
+        options.plotstyles_mpl = [options.mpl for i in xrange(nhists)]
     if not options.drawcommands_root:
         if options.stack:
             options.drawcommands_root = ['stack ' + options.draw
-                                         for i in xrange(nfiles)]
+                                         for i in xrange(nhists)]
         else:
-            options.drawcommands_root = [options.draw for i in xrange(nfiles)]
+            options.drawcommands_root = [options.draw for i in xrange(nhists)]
     if not options.markersizes:
         if options.markers:
             if options.mpl: size = matplotlib.rcParams['lines.markersize']
             else: size = ROOT.gStyle.GetMarkerSize()
         else: size = 0
-        options.markersizes = [size for i in xrange(nfiles)]
+        options.markersizes = [size for i in xrange(nhists)]
     if not options.errorbarcolors_mpl:
-        options.errorbarcolors_mpl = [None for i in xrange(nfiles)]
+        options.errorbarcolors_mpl = [None for i in xrange(nhists)]
     if not options.alphas_mpl:
-        options.alphas_mpl = [options.alpha for i in xrange(nfiles)]
+        options.alphas_mpl = [options.alpha for i in xrange(nhists)]
     if options.efficiency:
         for i in range(len(options.drawcommands_root)):
             if options.drawcommands_root[i] != 'stack':
@@ -1143,7 +1170,7 @@ def process_options(options):
         options.draw = 'nostack hist'
         if not options.markersizes[i]:
             options.markersizes[i] = ROOT.gStyle.GetMarkerSize()
-        if nfiles == 2 and options.mc_color:
+        if nhists == 2 and options.mc_color:
             options.fillcolors[(i+1)%2] = options.mc_color
     for opt in ['linecolors', 'fillcolors', 'markercolors',
                 'errorbarcolors_mpl']:
@@ -1152,9 +1179,10 @@ def process_options(options):
                                for x in colors])
     return options
 
-def process_local_options(options, name):
+def process_local_options(options, path):
     """Turn on sticky options for a particular plot."""
     new_options = copy.copy(options)
+    name = os.path.basename(path)
     def apply_opt(opt):
         if options.sticky and opt in name:
             return True
@@ -1219,6 +1247,11 @@ Matplotlib: if you have the matplotlib python plotting library installed on your
     parser.add_option('--config', action='store_true',
                       help="do nothing but write a template configuration file "
                       "called rootplot_config.py")
+    parser.add_option('--compare', type='str', default="", metavar="TARGETS",
+                      help="instead of comparing several files, compare "
+                      "several histograms or directories within the same "
+                      "file; takes a comma-separated string of histogram or "
+                      "folder names")
     parser.add_option('-e', '--ext', default='png',
                       help="choose an output extension; default is png")
     parser.add_option('-m', '--markers', action='store_true', default=False,
@@ -1255,9 +1288,12 @@ Matplotlib: if you have the matplotlib python plotting library installed on your
                       help="set the maximum value of the y-axis")
     parser.add_option('--ymin', type='float', default=None,
                       help="set the minimum value of the y-axis")
-    parser.add_option('--normalize', metavar='VALUE', default=None,
-                      help="if integer, normalize to the VALUEth file "
-                      "(starting with 1); if float, scale by VALUE")
+    parser.add_option('--scale', metavar='VALUE', default=1., type="float",
+                      help="normalize all histograms by VALUE, or if a comma-"
+                      "separated list, normalize differently for each "
+                      "file/hist")
+    parser.add_option('--normalize', metavar='VALUE', type='int', default=0,
+                      help="normalize to the VALUEth file (starting with 1)")
     parser.add_option('--range', metavar='LOWxHIGH',
                       help="only use the specified data range in determining "
                       "the normalization")
@@ -1270,7 +1306,10 @@ Matplotlib: if you have the matplotlib python plotting library installed on your
     parser.add_option('--legend', default=1, metavar='LOC',
                       help="Place legend in LOC, according to matplotlib "
                       "location codes; examples include 'upper right', "
-                      "'center', or 'center left'")
+                      "'center', or 'center left'; to turn off, set to 'None'")
+    parser.add_option('--legend-entries', default=None, metavar='LOC',
+                      help="A comma-separated string giving the labels to "
+                      "appear in the legend")
     parser.add_option('--title', default=None,
                       help="Replace the plot titles, or add to them by "
                       "preceeding with a '+'")
@@ -1337,7 +1376,7 @@ Matplotlib: if you have the matplotlib python plotting library installed on your
     group2.add_option('-f', '--fill', action='store_true', default=False,
                       help="Histograms will have a color fill")
     group3 = optparse.OptionGroup(parser,
-                                "Producing output with matplotlib")
+                                  "Producing output with matplotlib")
     group3.add_option('--mpl', action="store_true", default=False,
                       help="produce output in matplotlib; automatically "
                       "turned on by --errorbar, --bar, or --hist")
@@ -1385,20 +1424,20 @@ Matplotlib: if you have the matplotlib python plotting library installed on your
             print "         filenames on the command line will be ignored."
     else:
         options.filenames = filenames_from_interface
-    files = open_rootfiles(options.filenames)
     ## if no filenames provided, just display the help message
     if len(options.filenames) == 0:
         parser.print_help()
         sys.exit(0)
     options = process_options(options)
-    plots_to_make = process_directory('', files, options)
-    options.nplots = len(plots_to_make)
+    files = open_rootfiles(options)
+    recipes = get_plot_recipes(files, options)
+    options.nplots = len(recipes)
     if use_multiprocessing:
         queue = multi.JoinableQueue()
         queue_globals = multi.Manager().Namespace()
         queue_globals.nfinished = 0
         qargs = (queue, queue_globals, options)
-        for args in plots_to_make:
+        for args in recipes:
             queue.put(args)
         for i in range(options.processors):
             p = multi.Process(target=plotting_worker,
@@ -1407,26 +1446,22 @@ Matplotlib: if you have the matplotlib python plotting library installed on your
             p.start()
         queue.join()
     else:
-        for i, args in enumerate(plots_to_make):
+        for i, args in enumerate(recipes):
             process_hist(*args, **{'options':options})
             report_progress(i, options.nplots, options.output, options.ext)
     report_progress(options.nplots, options.nplots, options.output, options.ext)
     print ''
+    ## remove empty subdirectories
+    for root, dirs, files in os.walk(options.output):
+        if root != options.output and not dirs and not files:
+            os.rmdir(root)
+    ## add index.html files to all directories
     if options.ext in ['png', 'gif']:
-        try:
-            set
-        except NameError:
-            import sets
-        dirs = set([''])
-        for path, name, counter in plots_to_make:
-            while path.rfind('/') != -1:
-                dirs.add(path)
-                path = path[:path.rfind('/')]
-        for directory in dirs:
-            make_html_index(options.output, directory, options.ext,
+        for root, dirs, files in os.walk(options.output):
+            make_html_index(root, dirs, files, options.ext,
                             options.stylesheet, options.ncolumns_html)
     if options.merge:
-        merge_pdf(options.output, plots_to_make)
+        merge_pdf(options.output, recipes)
 
 
 if __name__ == '__main__':
