@@ -274,7 +274,7 @@ def tree2hists_main(config_file):
     print "\nScaled & added histograms from %i TTrees saved in\n  %s" % (len(list_of_files), output_filename)
 ##############################################################################
 
-if __name__ == "__main__":
+def main():
     if len(sys.argv) > 1:
         if path.isfile(sys.argv[1]):
             config_file = sys.argv[1].split('.')[0]
@@ -290,9 +290,7 @@ if __name__ == "__main__":
             print "tree2hists.py t2h_config.py"
             sys.exit(1)
         write_default_T2H_config()
-        sys.exit(0)
-    #import cProfile
-    #cProfile.run('main()', 'fooprof')
-    #import pstats
-    #p = pstats.Stats('fooprof')
-    #p.sort_stats('cumulative').print_stats(15)
+        sys.exit(0)    
+
+if __name__ == "__main__":
+    main()
